package hu.stunts.scr.apps;

import hu.stunts.scr.utils.InfoFormatter;
import hu.stunts.scr.utils.TxtParsing;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.*;

public class FlatDirReplaySet<T extends BaseReplay> implements Race{
	
	private final ClassLoader ccl = Thread.currentThread().getContextClassLoader();
	private URL urlToRacerNames = ccl.getResource("resources/racernames.txt");
	
	private Class<T> rplClass;
	private String raceTitle;
	private File rootDir;
	private RplDirectory rplDir;
	private int keyStart;
	private int keyEnd;
	private boolean performNameLookup = true;
	private InfoFormatter rFormatter = null;
	private Map<String, String> namesDict = null;
	private Map<String, String> bonusDict = null;
	private Map<String, String> penaltyDict = null;
	
	private List<T> activeReplays = new ArrayList<T>();

	private class MetaRacer extends AbstractInnerRacer<T> {
			
		public T genericReplay (String fileName, Map<String, String>... dictionaries) {
			return genericReplay(rplClass, fileName, dictionaries);
		}
		
		public boolean isLegal (T replay) {
			boolean answer = false;
			answer = (replay.isValid() &&
					replay.checkTermination());
			return answer;
		} 
	}
	
	public static <T extends BaseReplay> FlatDirReplaySet<T> create(File dir, Class<T> replayClass, int keyStart, int keyEnd, boolean lookName) {
		return new FlatDirReplaySet<T>(dir, replayClass, keyStart, keyEnd, lookName);
	}
	
	
	protected FlatDirReplaySet (File dir, Class<T> replayClass, int keyStart, int keyEnd, boolean lookName) {
		rplClass = replayClass;
		rootDir = dir;
		rplDir = new RplDirectory(rootDir);
		rFormatter = new InfoFormatter(dir);
		raceTitle = rootDir.getName();
		//bonus list
		int dictInitSize = 50;	
		if (rplClass.isAssignableFrom(BonusTableReplay.class)) {
			File bonusASCII = new File(rootDir, "carbonus.txt");
			if (bonusASCII.exists())
				bonusDict = TxtParsing.simpleMakeMapFromPairs(bonusASCII, dictInitSize);
		}
		//penalty list
		File penaltyASCII = new File(rootDir, "penalty.txt");
		if (penaltyASCII.exists())
			penaltyDict = TxtParsing.simpleMakeMapFromPairs(penaltyASCII, dictInitSize);
		//names list
		dictInitSize = 100;
		try {
			InputStream namesIS = urlToRacerNames.openStream();
			namesDict = TxtParsing.simpleMakeMapFromPairs(namesIS, dictInitSize, true);
		} catch (IOException e) {
			e.printStackTrace();
		}
		File extraNamesASCII = new File(rootDir, "racernames.txt");
		if (extraNamesASCII.exists()) {
			Map<String, String> extraNamesDict = TxtParsing.simpleMakeMapFromPairs(extraNamesASCII, dictInitSize, true);
			namesDict.putAll(extraNamesDict);
		}
		this.keyStart = keyStart;
		this.keyEnd = keyEnd;
		performNameLookup = lookName;
	}
	
	private String retrieveNameKey (File f) {
		int begin, end;
		String key = "";
		String fileName = f.getName().substring(0, f.getName().length() - 4);		begin = keyStart;
		end = keyEnd;
		if (end > fileName.length())
			end = fileName.length();
		if (!(begin < fileName.length()) || end == begin || fileName.length() < end - begin)
			key = fileName.substring(0, fileName.length() - 4);
		else
			key = fileName.substring(begin, end);
		return key;
	}
	
	private class NonRaceComparator implements Comparator<BaseReplay> {

		public int compare(BaseReplay arg0, BaseReplay arg1) {
			int answer = arg0.getTrack().compareTo(arg1.getTrack());
			if (answer == 0) {
				answer = arg0.compareTo(arg1);
			}		
			return answer;
		}
	}
	
	public void loadActiveReplays() {
		T currReplay = null;
		String racerName = "";
		String nameKey = "";
		List<File> rplTempList = new LinkedList<File>();
		activeReplays.clear();
		if (rplDir.getRplCount() > 0)
			for (int i = 0; i < rplDir.getRplCount(); i++)
				rplTempList.add(new File(rootDir, rplDir.getFileName(i)));
		ListIterator<File> it = rplTempList.listIterator();
		MetaRacer mRacer = new MetaRacer();
		while (it.hasNext()) {
			File currFile = it.next();
			nameKey =retrieveNameKey(currFile);
			currReplay = mRacer.genericReplay(currFile.getPath(), penaltyDict, bonusDict);
			if (currReplay != null && mRacer.isLegal(currReplay)) {
				if (performNameLookup)
					racerName = namesDict.get(nameKey);
				if (!performNameLookup || racerName == null)
					racerName = nameKey;
				if (currReplay != null)
					currReplay.setRacerName(racerName);
				activeReplays.add(currReplay);
			}
		}
		Collections.sort(activeReplays, new NonRaceComparator());
	}
	
	private class ScoreboardGenerator extends AbstractScoreboardGenerator {
		public ScoreboardGenerator() {
			super(activeReplays, rFormatter);
		}
	}
	
	public String[][] generateHIGStrings (boolean useCorrectedTimes) {
		ScoreboardGenerator sg = new ScoreboardGenerator();
		return sg.generateHIGStrings(useCorrectedTimes);
	}
	
	public void generateHIG (File dest, boolean correctTimes) {
		ScoreboardGenerator sg = new ScoreboardGenerator();
		sg.writeHIG(dest, correctTimes);
	}
	
	public String getRaceTitle() {
		return raceTitle;
	}
	
	public File getRootDir() {
		return rootDir;
	}
	
	public void printTest () {
		for (T replay : activeReplays) {
			System.out.println(replay.getRacer()+" : "+replay.toSeconds(replay.computeTime()));
		}
	}
	
}
